package cn.gygxzc.envir.config.shiro.cache

import cn.gygxzc.envir.sys.config.properties.ShiroRedisCacheProperties
import cn.gygxzc.envir.sys.config.properties.ShiroSessionProperties
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.DependsOn
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer
import org.springframework.data.redis.serializer.StringRedisSerializer

/**
 * @program: Zen-Framework
 *
 * @description: ${description}
 *
 * @author: tuonina
 *
 * @create: 2018-09-20 21:41
 **/
@Configuration
@EnableConfigurationProperties(value = [ShiroRedisCacheProperties::class, ShiroSessionProperties::class])
class RedisConfiguration {

    @Autowired
    private lateinit var redisProperties: ShiroRedisCacheProperties

    @Autowired
    private lateinit var sessionProperties: ShiroSessionProperties

    @Bean("shiroRedisConnectionFactory")
    fun connectionFactory(): JedisConnectionFactory {
        return JedisConnectionFactory(redisProperties)
    }


    @Bean(name = ["shiroRedisTemplate"])
    fun redisTemplate(): RedisTemplate<String, Any> {
        return RedisTemplate<String, Any>().apply {
            connectionFactory = connectionFactory()
            val stringSerializer = StringRedisSerializer()
            keySerializer = stringSerializer
            valueSerializer = JdkSerializationRedisSerializer()
            hashKeySerializer = stringSerializer
            hashValueSerializer = Jackson2JsonRedisSerializer(Any::class.java)
        }
    }


    @Bean(name = ["shiroRedisCacheManager"])
    @DependsOn(value = ["shiroRedisTemplate"])
    fun redisCacheManager(): ShiroRedisCacheManager {
        return ShiroRedisCacheManager(redisTemplate(), sessionProperties)
    }


}